
#include "main.h"
#include "editor_window.h"
#include "wad_manager.h"
#include "info_bar.h"
#include "map.h"
#include "prefs_dialog.h"
#include "3dmode.h"
#include "console_window.h"
#include "script_editor.h"
#include "tex_browser.h"
#include "edit_misc.h"
#include "checks.h"
#include "input.h"
#include "ttype_select.h"
#include "edit.h"
#include "sidebar.h"

#include <wx/image.h>

Wad	*edit_wad = NULL;
ScriptEditor *script_editor = NULL;

CVAR(Int, show_sidebar, 0, CVAR_SAVE)
CVAR(String, path_doomexe, "", CVAR_SAVE)
CVAR(String, run_parameters, "-file %a", CVAR_SAVE)

extern int edit_mode, gridsize;
extern int hilight_item;
extern point2_t mouse, down_pos;
extern WadManager *wad_manager;
extern Map map;
extern ConsoleWindow *console_window;
extern bool lock_hilight;
extern vector<string> valid_map_names;
extern string bg_tex, def_doortex, app_path;
extern thing_t last_thing;
extern WadList wads;

EXTERN_CVAR(Bool, edit_snap_grid)

EditorWindow::EditorWindow(const wxChar *title, int xpos, int ypos, int width, int height)
    : wxFrame((wxFrame *) NULL, -1, title, wxPoint(xpos, ypos), wxSize(width, height))
{
	// Setup menu
	setup_menu();

	// Setup toolbar
	setup_toolbar();

	wxBoxSizer *vbox = new wxBoxSizer(wxVERTICAL);
	SetSizer(vbox);

	wxBoxSizer *hbox = new wxBoxSizer(wxHORIZONTAL);
	vbox->Add(hbox, 1, wxEXPAND);

	// Map area
	map_area = new MapCanvas(this);
	hbox->Add(map_area, 1, wxEXPAND);
	map_area->set_menus(line_context_menu, sector_context_menu, thing_context_menu);

	// Sidebar
	sb_draw = new DrawSideBar(this);
	hbox->Add(sb_draw, 0, wxEXPAND);

	sb_theme = new ThemeSideBar(this);
	hbox->Add(sb_theme, 0, wxEXPAND);

	// Info bar
	info_bar = new LineInfoBar(this);
	vbox->Add(info_bar, 0, wxEXPAND);

	CreateStatusBar(4);
	int widths[] = { -1, 80, 80, 96 };
	SetStatusWidths(4, widths);

	SetSizeHints(800, 600, -1, -1);

	SetIcon(wxIcon("res/slade.ico",  wxBITMAP_TYPE_ICO, -1, -1));
	Maximize(true);
	Show();

	if (show_sidebar != 1)
		sb_draw->Show(false);
	if (show_sidebar != 2)
		sb_theme->Show(false);

	Layout();
}

EditorWindow::~EditorWindow()
{
}

void EditorWindow::setup_toolbar()
{
	CreateToolBar();
	wxToolBar *tb = GetToolBar();

	tb->SetToolBitmapSize(wxSize(16, 17));

	tb->AddTool(EWMENU_FILE_WADMANAGER, "Wad Manager", wxBitmap(wxImage("res/tb_temp16.png", wxBITMAP_TYPE_PNG)), _T("Wad Manager"));
	tb->AddSeparator();
	tb->AddTool(EWMENU_FILE_SAVE, "Save Map", wxBitmap(wxImage("res/tb_save16.png", wxBITMAP_TYPE_PNG)), _T("Save Map"));
	tb->AddTool(EWMENU_FILE_SAVEAS, "Save Map As", wxBitmap(wxImage("res/tb_saveas16.png", wxBITMAP_TYPE_PNG)), _T("Save Map As"));
	tb->AddTool(EWMENU_FILE_RUN, "Run Map", wxBitmap(wxImage("res/tb_run16.png", wxBITMAP_TYPE_PNG)), _T("Run Map"));
	tb->AddTool(EWMENU_FILE_CLOSE, "Close", wxBitmap(wxImage("res/tb_close16.png", wxBITMAP_TYPE_PNG)), _T("Close Map"));
	tb->AddSeparator();
	tb->AddRadioTool(EWMENU_MODE_VERTS, "Vertices Mode", wxBitmap(wxImage("res/tb_verts16.png", wxBITMAP_TYPE_PNG)),
						wxBitmap(wxImage("res/tb_verts16.png", wxBITMAP_TYPE_PNG)), _T("Vertices Mode"));
	tb->AddRadioTool(EWMENU_MODE_LINES, "Lines Mode", wxBitmap(wxImage("res/tb_lines16.png", wxBITMAP_TYPE_PNG)),
						wxBitmap(wxImage("res/tb_lines16.png", wxBITMAP_TYPE_PNG)), _T("Lines Mode"));
	tb->AddRadioTool(EWMENU_MODE_SECTORS, "Sectors Mode", wxBitmap(wxImage("res/tb_sectors16.png", wxBITMAP_TYPE_PNG)),
						wxBitmap(wxImage("res/tb_sectors16.png", wxBITMAP_TYPE_PNG)), _T("Sectors Mode"));
	tb->AddRadioTool(EWMENU_MODE_THINGS, "Things Mode", wxBitmap(wxImage("res/tb_things16.png", wxBITMAP_TYPE_PNG)),
						wxBitmap(wxImage("res/tb_things16.png", wxBITMAP_TYPE_PNG)), _T("Things Mode"));
	tb->AddSeparator();
	tb->AddTool(EWMENU_MODE_3D, "3D Mode", wxBitmap(wxImage("res/tb_3d16.png", wxBITMAP_TYPE_PNG)), _T("3d Mode"));
	tb->AddSeparator();
	tb->AddTool(EWMENU_EDIT_MIRRORX, "Mirror X", wxBitmap(wxImage("res/tb_mirrorx16.png", wxBITMAP_TYPE_PNG)), _T("Mirror X"));
	tb->AddTool(EWMENU_EDIT_MIRRORY, "Mirror Y", wxBitmap(wxImage("res/tb_mirrory16.png", wxBITMAP_TYPE_PNG)), _T("Mirror Y"));
	tb->AddTool(EWMENU_EDIT_ROTATE, "Rotate", wxBitmap(wxImage("res/tb_rotate16.png", wxBITMAP_TYPE_PNG)), _T("Rotate"));

	tb->ToggleTool(EWMENU_MODE_LINES, true);
	tb->Realize();
}

void EditorWindow::setup_menu()
{
	wxMenuBar *menu_bar = new wxMenuBar();
	SetMenuBar(menu_bar);

	// File menu
	wxMenu *file_menu = new wxMenu();
	file_menu->Append(EWMENU_FILE_WADMANAGER, "&Wad Manager", "Open the Wad Manager window");
	file_menu->AppendSeparator();
	file_menu->Append(EWMENU_FILE_RUN, "&Run Map", "Run the current wads/map with a doom executable");
	file_menu->Append(EWMENU_FILE_RENAME, "&Rename Map", "Rename the current map");
	file_menu->Append(EWMENU_FILE_IMPORT, "&Import Map", "Import a map from another wad file");
	file_menu->AppendSeparator();
	file_menu->Append(EWMENU_FILE_SAVE, "&Save", "Save the current map");
	file_menu->Append(EWMENU_FILE_SAVEAS, "Save &As", "Save the current wad to a new file");
	file_menu->Append(EWMENU_FILE_SAVESTANDALONE, "Save Standalone &Map", "Save the current map to a new wad file");
	file_menu->Append(EWMENU_FILE_CLOSE, "&Close", "Close the current map");
	file_menu->AppendSeparator();
	file_menu->Append(EWMENU_FILE_EXIT, "E&xit", "Exit SLADE");
	menu_bar->Append(file_menu, "&File");

	// Edit menu
	wxMenu *edit_menu = new wxMenu();

	// Line edit
	//line_edit_menu = new wxMenu();
	wxMenu *line_edit_menu = new wxMenu();
	line_edit_menu->Append(EWMENU_EDIT_ALIGNX, "Align &X", "Align selected lines' textures along the x axis");
	line_edit_menu->Append(EWMENU_EDIT_CORRECTREFS, "Correct Sector &References", "Attempts to set the correct sector references for the line");
	line_edit_menu->Append(EWMENU_EDIT_SPLITLINE, "&Split Line", "Split selected line(s)");
	line_edit_menu->Append(EWMENU_EDIT_EXTRUDELINE, "&Extrude Line", "'Extrudes' selected lines");
	edit_menu->Insert(0, EWMENU_EDIT_LINE, "&Line", line_edit_menu);

	// Line context
	line_context_menu = new wxMenu();
	line_context_menu->Append(EWMENU_EDIT_PROPERTIES, "&Properties", "Edit the line's properties");
	line_context_menu->AppendSeparator();
	line_context_menu->Append(EWMENU_EDIT_MIRRORX, "Mirror &X", "Mirror selected lines on the X axis");
	line_context_menu->Append(EWMENU_EDIT_MIRRORY, "Mirror &Y", "Mirror selected lines on the Y axis");
	line_context_menu->Append(EWMENU_EDIT_ROTATE, "&Rotate", "Rotate selected lines");
	line_context_menu->AppendSeparator();
	line_context_menu->Append(EWMENU_EDIT_ALIGNX, "Align &X", "Align selected lines' textures along the x axis");
	line_context_menu->Append(EWMENU_EDIT_CORRECTREFS, "Correct Sector &References", "Attempts to set the correct sector references for the line");
	line_context_menu->Append(EWMENU_EDIT_SPLITLINE, "&Split Line", "Split selected line(s)");
	line_context_menu->Append(EWMENU_EDIT_EXTRUDELINE, "&Extrude Line", "'Extrudes' selected lines");
	line_context_menu->Append(EWMENU_EDIT_FLIPLINE, "&Flip Line", "Flips the selected lines");

	// Sector edit
	wxMenu *sector_edit_menu = new wxMenu();
	sector_edit_menu->Append(EWMENU_EDIT_MERGESECTORS, "&Merge Sectors", "Merge selected sectors (don't delete any lines)");
	sector_edit_menu->Append(EWMENU_EDIT_JOINSECTORS, "&Join Sectors", "Join selected sectors (delete redundant lines)");
	sector_edit_menu->AppendSeparator();
	sector_edit_menu->Append(EWMENU_EDIT_CREATEDOOR, "Create &Door", "Create a door from the selected sector(s)");
	sector_edit_menu->Append(EWMENU_EDIT_CREATESTAIRS, "Create &Stairs", "Create stairs from the selected sectors");
	sector_edit_menu->Append(EWMENU_EDIT_LIGHTGRADIENT, "&Light Gradient", "Create a smooth light gradient from the first selected sector to the last");
	edit_menu->Insert(1, EWMENU_EDIT_SECTOR, "&Sector", sector_edit_menu);

	// Sector context
	sector_context_menu = new wxMenu();
	sector_context_menu->Append(EWMENU_EDIT_PROPERTIES, "&Properties", "Edit the sector's properties");
	sector_context_menu->AppendSeparator();
	sector_context_menu->Append(EWMENU_EDIT_MIRRORX, "Mirror &X", "Mirror selected sectors on the X axis");
	sector_context_menu->Append(EWMENU_EDIT_MIRRORY, "Mirror &Y", "Mirror selected sectors on the Y axis");
	sector_context_menu->Append(EWMENU_EDIT_ROTATE, "&Rotate", "Rotate selected sectors");
	sector_context_menu->AppendSeparator();
	sector_context_menu->Append(EWMENU_EDIT_CREATESECTOR, "&Create Sector", "Makes the area around the mouse pointer into a sector");
	sector_context_menu->Append(EWMENU_EDIT_MERGESECTORS, "&Merge Sectors", "Merge selected sectors (don't delete any lines)");
	sector_context_menu->Append(EWMENU_EDIT_JOINSECTORS, "&Join Sectors", "Join selected sectors (delete redundant lines)");
	sector_context_menu->AppendSeparator();
	sector_context_menu->Append(EWMENU_EDIT_CREATEDOOR, "Create &Door", "Create a door from the selected sector(s)");
	sector_context_menu->Append(EWMENU_EDIT_CREATESTAIRS, "Create &Stairs", "Create stairs from the selected sectors");
	sector_context_menu->Append(EWMENU_EDIT_LIGHTGRADIENT, "&Light Gradient", "Create a smooth light gradient from the first selected sector to the last");

	// Thing context
	thing_context_menu = new wxMenu();
	thing_context_menu->Append(EWMENU_EDIT_PROPERTIES, "&Properties", "Edit the thing's properties");
	thing_context_menu->Append(EWMENU_THING_INSERT, "&Insert Thing", "Insert a thing at the mouse position");
	thing_context_menu->AppendSeparator();
	thing_context_menu->Append(EWMENU_EDIT_MIRRORX, "Mirror &X", "Mirror selected things on the X axis");
	thing_context_menu->Append(EWMENU_EDIT_MIRRORY, "Mirror &Y", "Mirror selected things on the Y axis");
	thing_context_menu->Append(EWMENU_EDIT_ROTATE, "Rotate", "Rotate selected things");

	edit_menu->AppendSeparator();
	edit_menu->Append(EWMENU_EDIT_SCRIPTS, "&ACS Script Editor", "Open the ACS script editor");
	edit_menu->AppendSeparator();
	edit_menu->Append(EWMENU_EDIT_MIRRORX, "Mirror &X", "Mirror selected items on the X axis");
	edit_menu->Append(EWMENU_EDIT_MIRRORY, "Mirror &Y", "Mirror selected items on the Y axis");
	edit_menu->Append(EWMENU_EDIT_ROTATE, "&Rotate", "Rotates selected items");
	edit_menu->AppendSeparator();
	edit_menu->Append(EWMENU_EDIT_PREFS, "&Preferences", "Setup SLADE options");
	menu_bar->Append(edit_menu, "&Edit");

	// Mode menu
	wxMenu *mode_menu = new wxMenu();
	mode_menu->Append(EWMENU_MODE_VERTS, "&Vertices", "Vertex edit mode");
	mode_menu->Append(EWMENU_MODE_LINES, "&Lines", "Line edit mode");
	mode_menu->Append(EWMENU_MODE_SECTORS, "&Sectors", "Sector edit mode");
	mode_menu->Append(EWMENU_MODE_THINGS, "&Things", "Thing edit mode");
	mode_menu->Append(EWMENU_MODE_3D, "&3d Mode", "3d Mode");
	menu_bar->Append(mode_menu, "&Mode");

	// View menu
	wxMenu *view_menu = new wxMenu();

	wxMenu *view_sb_menu = new wxMenu();
	view_sb_menu->Append(EWMENU_VIEW_DRAWSIDEBAR, "&Drawing", "Shows the drawing options sidebar");
	view_sb_menu->Append(EWMENU_VIEW_THEMESIDEBAR, "&Theme", "Shows the theme options sidebar");

	view_menu->Insert(0, -1, "&Show Sidebar", view_sb_menu);
	view_menu->Append(EWMENU_VIEW_HIDESIDEBAR, "Hide &Sidebar", "Hides the SLADE sidebar");
	view_menu->AppendSeparator();
	view_menu->Append(EWMENU_VIEW_SETBGTEX, "Set &Background Texture", "Sets the background texture for the 2d edit view");
	view_menu->Append(EWMENU_VIEW_CLEARBGTEX, "Clear Background &Texture", "Clears the background texture");
	view_menu->AppendSeparator();
	view_menu->Append(EWMENU_VIEW_CONSOLE, "Show &Console", "Open the SLADE console window");
	menu_bar->Append(view_menu, "&View");

	// Checks menu
	wxMenu *checks_menu = new wxMenu();
	checks_menu->Append(EWMENU_CHECK_REMOVEVERTS, "Remove Free &Vertices", "Removes any vertices that aren't attached to a line");
	checks_menu->Append(EWMENU_CHECK_REMOVELINES, "Remove Zero-Length &Lines", "Removes any lines that have the same start and end vertex");
	checks_menu->Append(EWMENU_CHECK_REMOVESECTORS, "Remove Unused &Sectors", "Removes any sectors that aren't referenced by any sidedefs");
	checks_menu->Append(EWMENU_CHECK_REMOVESIDES, "Remove Unused Side&defs", "Removes any sidedefs that aren't referenced by any lines");
	checks_menu->Append(EWMENU_CHECK_TEXTURES, "Check &Textures", "Checks for any invalid textures");
	menu_bar->Append(checks_menu, "&Checks");
}

// open_map: Opens a map to edit, if wad is NULL then it's a new/standalone map
// ------------------------------------------------------------------------- >>
bool EditorWindow::open_map(Wad* wad, string mapname)
{
	if (wad)
	{
		if (!wad->get_lump(mapname, 0))
			map.create(mapname);
		else
		{
			if (!map.open(wad, mapname))
				return false;
		}

		SetTitle(wxString::Format("SLADE (%s, %s)", wad->path.c_str(), mapname.c_str()));
	}
	else
	{
		map.create(mapname);
		SetTitle(wxString::Format("SLADE (unsaved, %s)", mapname.c_str()));
	}

	redraw_map(true, true);
	edit_wad = wad;
	return true;
}

void EditorWindow::change_infobar()
{
	GetSizer()->Detach(info_bar);
	delete info_bar;

	if (edit_mode == 0)
	{
		info_bar = new VertexInfoBar(this);
		GetToolBar()->ToggleTool(EWMENU_MODE_VERTS, true);
	}
	else if (edit_mode == 1)
	{
		info_bar = new LineInfoBar(this);
		GetToolBar()->ToggleTool(EWMENU_MODE_LINES, true);
	}
	else if (edit_mode == 2)
	{
		info_bar = new SectorInfoBar(this);
		GetToolBar()->ToggleTool(EWMENU_MODE_SECTORS, true);
	}
	else if (edit_mode == 3)
	{
		info_bar = new ThingInfoBar(this);
		GetToolBar()->ToggleTool(EWMENU_MODE_THINGS, true);
	}
	else
		info_bar = new InfoBar(this);

	GetSizer()->Add(info_bar, 0, wxEXPAND);

	Layout();
}

void EditorWindow::update_statusbar()
{
	if (map.opened)
		SetStatusText(wxString::Format("(%d, %d)", (int)m_x(mouse.x), (int)-m_y(mouse.y)), 3);

	if (edit_snap_grid)
		SetStatusText(wxString::Format("Snap: ON"), 2);
	else
		SetStatusText(wxString::Format("Snap: OFF"), 2);

	SetStatusText(wxString::Format("Grid: %d", gridsize), 1);
}

void EditorWindow::draw_shape()
{
	((DrawSideBar*)sb_draw)->draw_shape();
}

BEGIN_EVENT_TABLE(EditorWindow, wxFrame)
	EVT_CLOSE(EditorWindow::close)

	// MENU EVENTS

	// File menu
	EVT_MENU(EWMENU_FILE_WADMANAGER,		EditorWindow::file_wadmanager)
	EVT_MENU(EWMENU_FILE_CLOSE,				EditorWindow::file_close)
	EVT_MENU(EWMENU_FILE_SAVE,				EditorWindow::file_save)
	EVT_MENU(EWMENU_FILE_SAVEAS,			EditorWindow::file_saveas)
	EVT_MENU(EWMENU_FILE_SAVESTANDALONE,	EditorWindow::file_savestandalone)
	EVT_MENU(EWMENU_FILE_RENAME,			EditorWindow::file_rename)
	EVT_MENU(EWMENU_FILE_RUN,				EditorWindow::file_run)
	EVT_MENU(EWMENU_FILE_IMPORT,			EditorWindow::file_importmap)
	EVT_MENU(EWMENU_FILE_EXIT,				EditorWindow::file_exit)

	// Edit menu
	EVT_MENU(EWMENU_EDIT_SCRIPTS,		EditorWindow::edit_scripts)
	EVT_MENU(EWMENU_EDIT_PREFS,			EditorWindow::edit_prefs)
	EVT_MENU(EWMENU_EDIT_PROPERTIES,	EditorWindow::edit_properties)
	EVT_MENU(EWMENU_EDIT_MIRRORX,		EditorWindow::edit_mirrorx)
	EVT_MENU(EWMENU_EDIT_MIRRORY,		EditorWindow::edit_mirrory)
	EVT_MENU(EWMENU_EDIT_ROTATE,		EditorWindow::edit_rotate)

	EVT_MENU(EWMENU_EDIT_ALIGNX, EditorWindow::edit_alignx)
	EVT_MENU(EWMENU_EDIT_CORRECTREFS, EditorWindow::edit_correctrefs)
	EVT_MENU(EWMENU_EDIT_SPLITLINE, EditorWindow::edit_splitline)
	EVT_MENU(EWMENU_EDIT_EXTRUDELINE, EditorWindow::edit_extrudeline)
	EVT_MENU(EWMENU_EDIT_FLIPLINE, EditorWindow::edit_flipline)

	EVT_MENU(EWMENU_EDIT_CREATESECTOR, EditorWindow::edit_createsector)
	EVT_MENU(EWMENU_EDIT_MERGESECTORS, EditorWindow::edit_mergesectors)
	EVT_MENU(EWMENU_EDIT_JOINSECTORS, EditorWindow::edit_joinsectors)
	EVT_MENU(EWMENU_EDIT_CREATEDOOR, EditorWindow::edit_createdoor)
	EVT_MENU(EWMENU_EDIT_CREATESTAIRS, EditorWindow::edit_createstairs)
	EVT_MENU(EWMENU_EDIT_LIGHTGRADIENT, EditorWindow::edit_lightgradient)

	// Mode menu
	EVT_MENU(EWMENU_MODE_VERTS,		EditorWindow::mode_vertices)
	EVT_MENU(EWMENU_MODE_LINES,		EditorWindow::mode_lines)
	EVT_MENU(EWMENU_MODE_SECTORS,	EditorWindow::mode_sectors)
	EVT_MENU(EWMENU_MODE_THINGS,	EditorWindow::mode_things)
	EVT_MENU(EWMENU_MODE_3D,		EditorWindow::mode_3d)

	// View menu
	EVT_MENU(EWMENU_VIEW_DRAWSIDEBAR,	EditorWindow::view_drawsidebar)
	EVT_MENU(EWMENU_VIEW_THEMESIDEBAR,	EditorWindow::view_themesidebar)
	EVT_MENU(EWMENU_VIEW_HIDESIDEBAR,	EditorWindow::view_hidesidebar)
	EVT_MENU(EWMENU_VIEW_CONSOLE,		EditorWindow::view_console)
	EVT_MENU(EWMENU_VIEW_SETBGTEX,		EditorWindow::view_setbgtex)
	EVT_MENU(EWMENU_VIEW_CLEARBGTEX,	EditorWindow::view_clearbgtex)

	// Checks menu
	EVT_MENU(EWMENU_CHECK_REMOVEVERTS,		EditorWindow::check_removeverts)
	EVT_MENU(EWMENU_CHECK_REMOVELINES,		EditorWindow::check_removelines)
	EVT_MENU(EWMENU_CHECK_REMOVESECTORS,	EditorWindow::check_removesectors)
	EVT_MENU(EWMENU_CHECK_REMOVESIDES,		EditorWindow::check_removesides)
	EVT_MENU(EWMENU_CHECK_TEXTURES,			EditorWindow::check_checktextures)

	// Thing context menu
	EVT_MENU(EWMENU_THING_INSERT,	EditorWindow::thing_insert)
END_EVENT_TABLE()

void EditorWindow::close(wxCloseEvent &event)
{
	if (!event.CanVeto())
		wxTheApp->ExitMainLoop();
	else
	{
		// Exit stuff
		if (map.changed & MC_SAVE_NEEDED)
		{
			int result = wxMessageBox("Save changes?", "SLADE", wxYES_NO|wxCANCEL|wxICON_QUESTION);

			if (result == wxYES)
				file_save(wxCommandEvent());
			else if (result == wxCANCEL)
				return;
		}

		wxTheApp->ExitMainLoop();
	}
}

void EditorWindow::file_wadmanager(wxCommandEvent &event)
{
	wad_manager->Show();
}

void EditorWindow::file_close(wxCommandEvent &event)
{
	if (map.changed & MC_SAVE_NEEDED)
	{
		int result = wxMessageBox("Save changes?", "SLADE", wxYES_NO|wxCANCEL|wxICON_QUESTION);

		if (result == wxYES)
			file_save(wxCommandEvent());
		else if (result == wxCANCEL)
			return;
	}

	map.close();
	hilight_item = -1;
	redraw_map(true, true);
	SetTitle("SLADE");
}

void EditorWindow::file_exit(wxCommandEvent &event)
{
	Close();
}

void EditorWindow::file_save(wxCommandEvent &event)
{
	if (!map.opened)
		return;

	if (edit_wad)
	{
		if (!edit_wad->locked)
		{
			map.add_to_wad(edit_wad);
			edit_wad->save(true);
			map.changed = (map.changed & ~MC_SAVE_NEEDED);
			SetTitle(wxString::Format("SLADE (%s, %s)", edit_wad->path.c_str(), map.name.c_str()));
			add_recent_wad(edit_wad->path);
		}
	}
	else
		file_saveas(event);
}

void EditorWindow::file_saveas(wxCommandEvent &event)
{
	if (!map.opened)
		return;

	string filename = wxFileSelector("Save Wadfile", "", "", "*.wad", "Doom Wad Files (*.wad)|*.wad", wxSAVE|wxOVERWRITE_PROMPT);

	if (filename != "")
	{
		if (!edit_wad)
			edit_wad = new Wad();

		edit_wad->path = filename;
		map.add_to_wad(edit_wad);
		edit_wad->save(true);
		add_recent_wad(filename);
		wads.open_wad(edit_wad->path);

		map.changed = (map.changed & ~MC_SAVE_NEEDED);
		SetTitle(wxString::Format("SLADE (%s, %s)", edit_wad->path.c_str(), map.name.c_str()));
	}
}

void EditorWindow::file_savestandalone(wxCommandEvent &event)
{
	if (!map.opened)
		return;

	string filename = wxFileSelector("Save Standalone Map", "", "", "*.wad", "Doom Wad Files (*.wad)|*.wad", wxSAVE|wxOVERWRITE_PROMPT);

	if (filename != "")
	{
		edit_wad = new Wad();
		edit_wad->path = filename;
		map.add_to_wad(edit_wad);
		edit_wad->save(true);
		add_recent_wad(filename);

		map.changed = (map.changed & ~MC_SAVE_NEEDED);
		SetTitle(wxString::Format("SLADE (%s, %s)", edit_wad->path.c_str(), map.name.c_str()));
	}
}

void EditorWindow::file_rename(wxCommandEvent &event)
{
	if (edit_wad)
	{
		if (edit_wad->locked)
			return;
	}

	string mapname;
	if (valid_map_names.size() == 0)
	{
		wxString mname;
		mname = wxGetTextFromUser("Enter Map Name:", "Rename Map");
		mname.UpperCase();
		mname.Truncate(8);
		mapname = wx_to_str(mname);
	}
	else
	{
		wxArrayString mapnames;
		for (int a = 0; a < valid_map_names.size(); a++)
			mapnames.Add(valid_map_names[a].c_str());

		mapname = wxGetSingleChoice("Select Map Name", "Rename Map", mapnames);
	}

	if (mapname != "")
	{
		if (valid_map_names.size() == 0 || vector_exists(valid_map_names, mapname))
		{
			if (edit_wad)
			{
				if (vector_exists(edit_wad->available_maps, mapname))
				{
					wxMessageBox("Map already exists in the wad!", "Error", wxICON_ERROR);
					return;
				}
			}

			map.name = mapname;

			if (edit_wad)
				SetTitle(wxString::Format("SLADE (%s, %s)", edit_wad->path.c_str(), map.name.c_str()));
			else
				SetTitle(wxString::Format("SLADE (unsaved, %s)", edit_wad->path.c_str(), map.name.c_str()));
		}
		else
			wxMessageBox("Invalid map name!", "Error", wxICON_ERROR);
	}
}

void EditorWindow::file_run(wxCommandEvent &event)
{
	string exe_path = path_doomexe;
	if (exe_path == "")
	{
		wxMessageBox("No executable set. See edit->preferences->misc");
		return;
	}
	
	Tokenizer tz;
	tz.open_string(run_parameters, 0, 0);

	Wad *temp_wad = new Wad();
	map.add_to_wad(temp_wad);
	temp_wad->path = parse_string("%ssladetemp.wad", app_path.c_str());
	temp_wad->save(true);
	wads.open_wad(temp_wad->path);

	string token = tz.get_token();
	while (token != "!END")
	{
		exe_path += " ";

		if (token == "%w")
		{
			if (edit_wad)
				exe_path += parse_string("\"%s\" ", edit_wad->path.c_str());
		}

		else if (token == "%a")
		{
			for (int a = 0; a < wads.n_wads; a++)
				exe_path += parse_string("\"%s\" ", wads.get_wad(a)->path.c_str());
		}

		else if (token == "%m")
			exe_path += map.name;

		else if (token == "%i")
			exe_path += parse_string("\"%s\" ", wads.get_iwad()->path.c_str());

		else
			exe_path += token;

		token = tz.get_token();
	}

	//wxLogMessage(exe_path.c_str());

	wads.close_wad(temp_wad->path);

	if (wxExecute(exe_path.c_str()) == -1)
		wxMessageBox("Couldn't find executable! Check the path at edit->preferences->misc");
}

void EditorWindow::file_importmap(wxCommandEvent &event)
{
	string filename = "";
	wxFileDialog browse(NULL, "Open Wad", "", "", "Doom Wad Files (*.wad)|*.wad", wxOPEN|wxFILE_MUST_EXIST);
	if (browse.ShowModal() == wxID_OK)
		filename = browse.GetPath();

	if (filename != "")
	{
		Wad temp_wad;

		if (!temp_wad.open(filename))
		{
			wxMessageBox("Error Opening Wadfile!");
			return;
		}

		string mapname;
		wxArrayString mapnames;
		for (int a = 0; a < temp_wad.available_maps.size(); a++)
			mapnames.Add(temp_wad.available_maps[a].c_str());

		if (mapnames.size() == 0)
		{
			wxMessageBox("The wadfile doesn't have any maps");
			return;
		}

		mapname = wxGetSingleChoice("Select Map", "Import Map", mapnames);

		if (mapname == "")
			return;

		map.open(&temp_wad, mapname);
		redraw_map(true);
	}
}

void EditorWindow::edit_scripts(wxCommandEvent &event)
{
	if (!map.hexen)
		return;

	if (!script_editor)
		script_editor = new ScriptEditor(this);

	script_editor->Show();
}

void EditorWindow::edit_prefs(wxCommandEvent &event)
{
	PrefsDialog pd;
	pd.ShowModal();
}

void EditorWindow::edit_properties(wxCommandEvent &event)
{
	if (!map.opened)
		return;

	lock_hilight = true;
	edit_item();
	lock_hilight = false;
}

void EditorWindow::edit_mirrorx(wxCommandEvent &event)
{
	if (!map.opened)
		return;

	lock_hilight = true;

	if (edit_mode == 3)
		thing_mirror(true, false, true);
	else
		vert_mirror(true, false);

	lock_hilight = false;
}

void EditorWindow::edit_mirrory(wxCommandEvent &event)
{
	if (!map.opened)
		return;

	lock_hilight = true;

	if (edit_mode == 3)
		thing_mirror(false, true, true);
	else
		vert_mirror(false, true);

	lock_hilight = false;
}

void EditorWindow::edit_rotate(wxCommandEvent &event)
{
	if (!map.opened)
		return;

	lock_hilight = true;

	int ang = wxGetNumberFromUser("", "Enter degrees to rotate", "Rotate", 0, -360, 360, this);

	if (ang == -1)
	{
		lock_hilight = false;
		return;
	}
	else
	{
		if (edit_mode == 3)
			thing_rotate(ang, true);
		else
			vert_rotate(ang);

		lock_hilight = false;
	}
}

void EditorWindow::edit_alignx(wxCommandEvent &event)
{
	if (!map.opened)
		return;

	line_align_x();
}

void EditorWindow::edit_correctrefs(wxCommandEvent &event)
{
	if (!map.opened)
		return;

	lock_hilight = true;
	line_correct_references();
	lock_hilight = false;
}

void EditorWindow::edit_splitline(wxCommandEvent &event)
{
	if (!map.opened)
		return;

	lock_hilight = true;
	long splits = wxGetNumberFromUser("", "Enter Number of Splits:", "Split Line", 2, 2, 100);
	//if (splits = -1)
	//	return;

	wxLogMessage("Split line %d", splits);

	line_split(splits);
	lock_hilight = false;
}

void EditorWindow::edit_extrudeline(wxCommandEvent &event)
{
	if (!map.opened)
		return;

	lock_hilight = true;
	int dist = atoi(wxGetTextFromUser("Enter Distance to Extrude:", "Extrude Line", "8"));

	line_extrude(dist);
	lock_hilight = false;
}

void EditorWindow::edit_flipline(wxCommandEvent &event)
{
	if (!map.opened)
		return;

	lock_hilight = true;
	line_flip(true, true);
	lock_hilight = false;
	redraw_map(true, false);
}

void EditorWindow::edit_createsector(wxCommandEvent &event)
{
	if (!map.opened)
		return;

	//sector_merge(false);
	sector_create(point2_t(m_x(down_pos.x), -m_y(down_pos.y)));
}

void EditorWindow::edit_mergesectors(wxCommandEvent &event)
{
	if (!map.opened)
		return;

	sector_merge(false);
}

void EditorWindow::edit_joinsectors(wxCommandEvent &event)
{
	if (!map.opened)
		return;

	sector_merge(true);
}

void EditorWindow::edit_createdoor(wxCommandEvent &event)
{
	if (!map.opened)
		return;

	lock_hilight = true;
	string tex = "";

	TextureBrowser tb(TEXTURES_WALLS, def_doortex);
	if (tb.ShowModal() == wxID_OK)
		tex = tb.get_texture();
	else 
		return;

	sector_create_door(tex);
	lock_hilight = false;
}

void EditorWindow::edit_createstairs(wxCommandEvent &event)
{
	if (!map.opened)
		return;

	lock_hilight = true;
	// Custom dialog ahoy
	wxDialog dlg(NULL, -1, "Create Stairs", wxDefaultPosition, wxDefaultSize);
	wxBoxSizer *vbox = new wxBoxSizer(wxVERTICAL);
	dlg.SetSizer(vbox);
	wxTextCtrl *entry_floor = new wxTextCtrl(&dlg, -1, "8");
	wxTextCtrl *entry_ceil = new wxTextCtrl(&dlg, -1, "0");
	wxBoxSizer *hbox = new wxBoxSizer(wxHORIZONTAL);
	vbox->Add(hbox, 0, wxEXPAND|wxALL, 4);
	hbox->Add(new wxStaticText(&dlg, -1, "Floor Step:"), 1, wxRIGHT|wxALIGN_CENTER_VERTICAL, 4);
	hbox->Add(entry_floor, 0, wxEXPAND);
	hbox = new wxBoxSizer(wxHORIZONTAL);
	vbox->Add(hbox, 0, wxEXPAND|wxALL, 4);
	hbox->Add(new wxStaticText(&dlg, -1, "Ceiling Step:"), 1, wxRIGHT|wxALIGN_CENTER_VERTICAL, 4);
	hbox->Add(entry_ceil, 0, wxEXPAND);
	vbox->Add(dlg.CreateButtonSizer(wxOK|wxCANCEL), 0, wxEXPAND|wxALL, 4);
	dlg.SetBestFittingSize();

	if (dlg.ShowModal() == wxID_OK)
		sector_create_stairs(atoi(entry_floor->GetValue()), atoi(entry_ceil->GetValue()));

	lock_hilight = false;
}

void EditorWindow::edit_lightgradient(wxCommandEvent &event)
{
	if (!map.opened)
		return;

	// Custom dialog ahoy
	wxDialog dlg(NULL, -1, "Create Stairs", wxDefaultPosition, wxDefaultSize);
	wxBoxSizer *vbox = new wxBoxSizer(wxVERTICAL);
	dlg.SetSizer(vbox);
	wxCheckBox *cb_first_to_last = new wxCheckBox(&dlg, -1, "Use Existing Values");
	wxTextCtrl *entry_step = new wxTextCtrl(&dlg, -1, "16");
	wxBoxSizer *hbox = new wxBoxSizer(wxHORIZONTAL);
	vbox->Add(hbox, 0, wxEXPAND|wxALL, 4);
	hbox->Add(new wxStaticText(&dlg, -1, "Light Step:"), 1, wxRIGHT|wxALIGN_CENTER_VERTICAL, 4);
	hbox->Add(entry_step, 0, wxEXPAND);
	vbox->Add(cb_first_to_last, 0, wxEXPAND|wxALL, 4);
	vbox->Add(dlg.CreateButtonSizer(wxOK|wxCANCEL), 0, wxEXPAND|wxALL, 4);
	dlg.SetBestFittingSize();

	if (dlg.ShowModal() == wxID_OK)
	{
		if (cb_first_to_last->GetValue())
			sector_light_gradient(9999);
		else
			sector_light_gradient(atoi(entry_step->GetValue()));
	}
}

void EditorWindow::mode_vertices(wxCommandEvent &event)
{
	change_edit_mode(0);
	update_statusbar();
}

void EditorWindow::mode_lines(wxCommandEvent &event)
{
	change_edit_mode(1);
	update_statusbar();
}

void EditorWindow::mode_sectors(wxCommandEvent &event)
{
	change_edit_mode(2);
	update_statusbar();
}

void EditorWindow::mode_things(wxCommandEvent &event)
{
	change_edit_mode(3);
	update_statusbar();
}

void EditorWindow::mode_3d(wxCommandEvent &event)
{
	if (map.opened)
		start_3d_mode();
}

void EditorWindow::view_drawsidebar(wxCommandEvent &event)
{
	show_sidebar = 1;
	sb_draw->Show();
	sb_theme->Show(false);
	Layout();
}

void EditorWindow::view_themesidebar(wxCommandEvent &event)
{
	show_sidebar = 2;
	sb_draw->Show(false);
	sb_theme->Show();
	Layout();
}

void EditorWindow::view_hidesidebar(wxCommandEvent &event)
{
	show_sidebar = 0;
	sb_draw->Show(false);
	sb_theme->Show(false);
	Layout();
}

void EditorWindow::view_console(wxCommandEvent &event)
{
	console_window->Show();
}

void EditorWindow::view_setbgtex(wxCommandEvent &event)
{
	if (!map.opened)
		return;

	TextureBrowser tb(2);

	if (tb.ShowModal() == wxID_OK)
		bg_tex = tb.get_texture();

	redraw_map();
}

void EditorWindow::view_clearbgtex(wxCommandEvent &event)
{
	bg_tex = "-";
	redraw_map();
}

void EditorWindow::check_removeverts(wxCommandEvent &event)
{
	if (!map.opened)
		return;

	int v = remove_free_verts();
	wxMessageBox(wxString::Format("Removed %d Free Vertices", v));
}

void EditorWindow::check_removelines(wxCommandEvent &event)
{
	if (!map.opened)
		return;

	int l = remove_zerolength_lines();
	wxMessageBox(wxString::Format("Removed %d Zero-Length Lines", l));
}

void EditorWindow::check_removesectors(wxCommandEvent &event)
{
	if (!map.opened)
		return;

	int s = remove_unused_sectors();
	wxMessageBox(wxString::Format("Removed %d Unused Sectors", s));
}

void EditorWindow::check_removesides(wxCommandEvent &event)
{
	if (!map.opened)
		return;

	int s = remove_unused_sides();
	wxMessageBox(wxString::Format("Removed %d Unused Sidedefs", s));
}

void EditorWindow::check_checktextures(wxCommandEvent &event)
{
	if (!map.opened)
		return;

	check_textures();
}

void EditorWindow::thing_insert(wxCommandEvent &event)
{
	if (!map.opened)
		return;

	TTypeSelectDialog td(last_thing.type);

	if (td.ShowModal() == wxID_OK)
	{
		last_thing.type = td.get_type();
		create_thing();
		redraw_map(true, false);
		//map.add_thing(snap_to_grid(down_pos.x), snap_to_grid(down_pos.y), last_thing);
	}
}
