
#include "main.h"
#include "editor_window.h"
#include "map.h"
#include "wad_manager.h"
#include "game_config.h"
#include "splash.h"
#include "keybind.h"
#include "colours.h"
#include "console.h"

#include <wx/image.h>
#include <wx/sysopt.h>
#include <wx/tipdlg.h>
#include <wx/tooltip.h>
#include <wx/stdpaths.h>

WadManager		*wad_manager;
EditorWindow	*editor_window;

string app_path;

CVAR(Bool, reopen_wads, true, CVAR_SAVE)
CVAR(String, col_config, "Default", CVAR_SAVE)
CVAR(Bool, tips_start, 1, CVAR_SAVE)
CVAR(Int, tip_current, 0, CVAR_SAVE)

extern WadList wads;
extern Map map;
extern BindList binds;

string parse_string(char *str, ...)
{
	char text[512] = "";
	va_list ap;

	va_start(ap, str);
	vsprintf(text, str, ap);
	va_end(ap);

	string ret = text;
	return ret;
}

// str_upper: Returns a string in uppercase
// ------------------------------------- >>
string str_upper(string str)
{
	DWORD len = 0;
	char* ustr = NULL;

	for (DWORD c = 0; c < str.length(); c++)
		len++;

	ustr = (char *)malloc(len + 1);

	for (DWORD c = 0; c < str.length(); c++)
		ustr[c] = str[c];

	ustr[len] = 0;

	string ret = strupr(ustr);
	//string ret = g_ascii_strup(ustr, -1);
	free(ustr);
	return ret;
}

// wx_to_str: Converts a wxString to a std::string (also from unicode to ascii)
// ------------------------------------------------------------------------- >>
string wx_to_str(wxString str)
{
	string ret;

	wxCharBuffer buffer = str.ToAscii();
	const char* data = buffer;
	ret = data;

	return ret;
}

// load_main_config: Loads the main SLADE configuration file
// ------------------------------------------------------ >>
void load_main_config(bool wads_open)
{
	Tokenizer mr;

	if (!mr.open_file("slade.cfg", 0, 0))
		return;

	string token = mr.get_token();

	while (token != "!END")
	{
		if (token == "cvars")
			load_cvars(&mr);

		if (token == "iwads")
			load_game_iwads(&mr);

		if (token == "recent_wads")
			load_recent_wads(&mr);

		if (token == "open_wads" && reopen_wads && !wads_open)
			load_open_wads(&mr);

		if (token == "key_binds")
			binds.load(&mr);
		
		/*
		if (token == "window_props")
			load_window_properties(&mr);
			*/

		token = mr.get_token();
	}
}

// save_main_config: Saves the main SLADE configuration file
// ------------------------------------------------------ >>
void save_main_config()
{
	FILE *fp = fopen("slade.cfg", "wt");
	save_cvars(fp);
	save_game_iwads(fp);
	binds.save(fp);
	save_recent_wads(fp);
	if (reopen_wads) save_open_wads(fp);
	//save_window_properties(fp);
	fclose(fp);
}

IMPLEMENT_APP(MainApp)

bool MainApp::OnInit()
{
	srand(wxGetLocalTime());

	wxStandardPaths sp;
	wxSetWorkingDirectory(sp.GetDataDir());

	wxLog::SetActiveTarget(new wxLogStderr(fopen("slade.log", "wt")));
	wxSystemOptions::SetOption(wxT("msw.remap"), 0);
	wxSystemOptions::SetOption(wxT("msw.notebook.themed-background"), 0);
	wxImage::AddHandler(new wxPNGHandler);
	wxToolTip::Enable(true);

	app_path = wx_to_str(sp.GetDataDir());

#ifdef WIN32
	app_path += "\\";
#else
	app_path += "/";
#endif

	wxLogMessage("%s", app_path.c_str());

	setup_splash();
	splash("Starting Up...", false);

	bool wads_opened = false;
	if (argc > 0)
	{
		for (int a = 0; a < argc; a++)
			wxLogMessage(argv[a]);

		for (int a = 1; a < argc; a++)
		{
			wxString arg = argv[a];

			if (arg.Right(4) == _T(".wad") ||
				arg.Right(4) == _T(".WAD") ||
				arg.Right(4) == _T(".Wad"))
			{
				wxLogMessage("Opening wad %s", arg);
				wads.open_wad(wx_to_str(arg));
				wads_opened = true;
			}
		}
	}

	load_main_config(wads_opened);

	editor_window = new EditorWindow("SLADE", -1, -1, 800, 600);
	SetTopWindow(editor_window);

	init_console();

	load_colour_configs();
	string ccfg = col_config;
	set_colour_config(ccfg);

	wad_manager = new WadManager(editor_window);
	splash_hide();

	if (tips_start)
    {
		wxTipProvider *tipProvider = wxCreateFileTipProvider("res/tips.txt", tip_current);
		tips_start = wxShowTip(editor_window, tipProvider);
		tip_current = tipProvider->GetCurrentTip();
		delete tipProvider;
    }

	return true;
}

int MainApp::OnExit()
{
	save_main_config();

	return 0;
}
